/***************************************************************************
*   Copyright (C) 2003 by Sandro Giessl                                   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include <kconfig.h>
#include <kdemacros.h>

#include "flatknifty.h"
#include "flatkniftyclient.h"
#include "misc.h"

using namespace flatKNifty;

// static bool pixmaps_created = false;

bool flatKniftyHandler::m_initialized    = false;
bool flatKniftyHandler::m_useTitleColor  = true;
int  flatKniftyHandler::m_useHighContrastButtons = flatKniftyHandler::WhenHovered;
int  flatKniftyHandler::m_drawBordersOnButtons   = flatKniftyHandler::WhenHovered;
bool flatKniftyHandler::m_bordersBlendColor = true;
bool flatKniftyHandler::m_titleShadow    = false;
bool flatKniftyHandler::m_shrinkBorders  = true;
int  flatKniftyHandler::m_borderSize     = 4;
int  flatKniftyHandler::m_titleHeight    = 19;
int  flatKniftyHandler::m_titleHeightTool= 12;
QFont flatKniftyHandler::m_titleFont = QFont();
QFont flatKniftyHandler::m_titleFontTool = QFont();
Qt::AlignmentFlags flatKniftyHandler::m_titleAlign = Qt::AlignHCenter;

flatKniftyHandler::flatKniftyHandler()
{
    reset(1);
}

flatKniftyHandler::~flatKniftyHandler()
{
    m_initialized = false;
}

KDecoration* flatKniftyHandler::createDecoration (KDecorationBridge *b)
{
    m_titleFont = options()->font(true, false);
    return new flatKniftyClient(b, this);
}

bool flatKniftyHandler::reset(unsigned long changed)
{
    m_titleFont = options()->font(true, false);
    m_titleFontTool = m_titleFont;
    m_titleFontTool.setPointSize(m_titleFontTool.pointSize() - 3); // Shrink font by 3pt
    m_titleFontTool.setWeight( QFont::Normal ); // and disable bold

    // read in the configuration
    m_initialized = false;
    readConfig();

    // reset all clients
    resetDecorations (changed);

    m_initialized = true;

    return true;
}

bool flatKniftyHandler::readConfig()
{
    // create a config object
    KConfig config("kwinflatkniftyrc");
    config.setGroup("General");

    // grab settings
    m_titleShadow    = config.readBoolEntry("TitleShadow", false);
    m_shrinkBorders  = config.readBoolEntry("ShrinkBorders", true);
    switch(KDecoration::options()->preferredBorderSize( this )) {
        case BorderTiny:
            m_borderSize = 2;
            break;
        case BorderLarge:
            m_borderSize = 6;
            break;
        case BorderVeryLarge:
            m_borderSize = 10;
            break;
        case BorderHuge:
            m_borderSize = 18;
            break;
        case BorderVeryHuge:
            m_borderSize = 27;
            break;
        case BorderOversized:
            m_borderSize = 40;
            break;
        case BorderNormal:
        default:
            m_borderSize = 5;
    }

    QFontMetrics fm(m_titleFont);  // active font = inactive font
    int titleHeightMin = 16;
    // The title should strech with bigger font sizes!
    m_titleHeight = QMAX(titleHeightMin, fm.height() + 4); // 4 px for the shadow etc.

    fm = QFontMetrics(m_titleFontTool);  // active font = inactive font
    int titleHeightToolMin = 13;
    // The title should strech with bigger font sizes!
    m_titleHeightTool = QMAX(titleHeightToolMin, fm.height() ); // don't care about the shadow etc.

    QString value = config.readEntry("TitleAlignment", "AlignHCenter");
    if (value == "AlignLeft")         m_titleAlign = Qt::AlignLeft;
    else if (value == "AlignHCenter") m_titleAlign = Qt::AlignHCenter;
    else if (value == "AlignRight")   m_titleAlign = Qt::AlignRight;

    m_useHighContrastButtons = config.readNumEntry("UseHighContrastButtons", WhenHovered);
    m_drawBordersOnButtons = config.readNumEntry("DrawBordersOnButtons", WhenHovered );
    m_useTitleColor = config.readBoolEntry("UseTitleBarColor", true);
    m_bordersBlendColor = config.readBoolEntry("BordersBlendColor", false);

    return true;
}

//////////////////////////////////////////////////////////////////////////////
// Plugin Stuff                                                             //
//////////////////////////////////////////////////////////////////////////////

static flatKniftyHandler *handler = 0;

extern "C"
{
    KDE_EXPORT KDecorationFactory* create_factory()
    {
        return new flatKniftyHandler();
    }
}

// flatKniftyHandler* clientHandler()
// {
//     return handler;
// }
